using System;
using System.Drawing;
using System.ComponentModel;
using System.Windows.Forms;
using Infragistics.Win;
using Infragistics.Win.UltraWinGrid;

namespace UltraWinGrid_ActiveRow_Border_CS
{
	/// <summary>
	/// Summary description for ActiveBorderDrawFilter.
	/// </summary>
	public class ActiveBorderDrawFilter : IUIElementDrawFilter, IDisposable
	{
		#region Private Members

		private Color		activeRowBorderColor = Color.Red;
		private Color		activeCellBorderColor = Color.Blue;

		// This class will need a pointer back to the grid in order to get
		// the ActiveCell and/or ActiveRow. 
		private UltraGrid	grid;

		#endregion Private Members

		#region Constructors

		public ActiveBorderDrawFilter(UltraGrid grid) : this(grid, Color.Empty, Color.Empty) {}

		public ActiveBorderDrawFilter(UltraGrid grid, Color activeRowBorderColor) : this (grid, activeRowBorderColor, Color.Empty) {}

		public ActiveBorderDrawFilter(UltraGrid grid, Color activeRowBorderColor, Color activeCellBorderColor)
		{
			this.grid = grid;

			// When a cell loses focus, it doesn't neccessarily get invalidated. So
			// we need to hook BeforeCellDeactivate and dirty the cell so 
			// the border goes away when the cell loses focus. 
			this.grid.BeforeCellDeactivate += new CancelEventHandler(this.grid_BeforeCellDeactivate);

			this.activeRowBorderColor = activeRowBorderColor;
			this.activeCellBorderColor = activeCellBorderColor;
		}
		#endregion Constructors	

		#region Public Properties

		#region ActiveRowBorderColor
		public Color ActiveRowBorderColor
		{
			get { return this.activeRowBorderColor; }
			set 
			{
				this.activeRowBorderColor = value;
			}
		}
		#endregion ActiveRowBorderColor

		#region ActiveCellBorderColor
		public Color ActiveCellBorderColor
		{
			get { return this.activeCellBorderColor; }
			set 
			{
				this.activeCellBorderColor = value;
			}
		}
		#endregion ActiveCellBorderColor

		#endregion Public Properties

		#region Event Sinks

		#region grid_BeforeCellDeactivate
		public void grid_BeforeCellDeactivate (object sender, CancelEventArgs e)
		{
			// The previously active cell doesn't always get repainted when it
			// loses focus. So we need to make sure it's invalidated so that 
			// the border goes away. 
			if ( ! this.activeCellBorderColor.IsEmpty &&
				this.activeCellBorderColor != Color.Transparent )
			{
				// Find the ActiveCell
				UltraGridCell activeCell = this.grid.ActiveCell;
				if (activeCell != null)
				{
					// The ActiveCell may be displayed in multiple scroll regions. So 
					// we need to loop through all of them. 
					foreach (RowScrollRegion rsr in this.grid.DisplayLayout.RowScrollRegions)
					{
						foreach (ColScrollRegion csr in this.grid.DisplayLayout.ColScrollRegions)
						{
							UIElement activeCellUIElement = activeCell.GetUIElement(rsr, csr);
							if (activeCellUIElement != null)
							{								
								Rectangle activeCellRect = activeCellUIElement.Rect;
								// The cell is within the row, so we need to extend the rect a little
								// bit vertically. 
								activeCellRect.Inflate(0, 1);

								this.grid.Invalidate(activeCellRect);
							}
						}
					}
				}
			}
		}
		
		#endregion grid_BeforeCellDeactivate

		#endregion Event Sinks

		#region Implementation of IUIElementDrawFilter

		Infragistics.Win.DrawPhase IUIElementDrawFilter.GetPhasesToFilter(ref Infragistics.Win.UIElementDrawParams drawParams)
		{
			RowColRegionIntersectionUIElement rowColRegionIntersectionUIElement = drawParams.Element as RowColRegionIntersectionUIElement;
			if (rowColRegionIntersectionUIElement == null)
				return DrawPhase.None;

			return DrawPhase.AfterDrawElement;
		}

		bool IUIElementDrawFilter.DrawElement(Infragistics.Win.DrawPhase drawPhase, ref Infragistics.Win.UIElementDrawParams drawParams)
		{
			// If the Row Border color is not empty or transparent, draw
			// a border around the ActiveRow.
			if ( ! this.activeRowBorderColor.IsEmpty &&
				this.activeRowBorderColor != Color.Transparent )
			{
				// Find the ActiveRow
				UltraGridRow activeRow = this.grid.ActiveRow;
				if (activeRow != null)
				{
					// The ActiveRow may be displayed in multiple scroll regions. So 
					// we need to loop through all of them. 
					foreach (RowScrollRegion rsr in this.grid.DisplayLayout.RowScrollRegions)
					{
						foreach (ColScrollRegion csr in this.grid.DisplayLayout.ColScrollRegions)
						{
							UIElement activeRowUIElement = activeRow.GetUIElement(rsr, csr);
							if (activeRowUIElement != null)
							{								
								Rectangle activeRowRect = activeRowUIElement.ClipRect;
								drawParams.DrawBorders(
									UIElementBorderStyle.Solid, 
									System.Windows.Forms.Border3DSide.All, 
									this.activeRowBorderColor, 
									this.activeRowBorderColor,
									activeRowRect, 
									drawParams.InvalidRect );								
							}
						}
					}
				}
			}

			// If the Cell Border color is not empty or transparent, draw
			// a border around the ActiveCell.
			if ( ! this.activeCellBorderColor.IsEmpty &&
				this.activeCellBorderColor != Color.Transparent )
			{
				// Find the ActiveCell
				UltraGridCell activeCell = this.grid.ActiveCell;
				if (activeCell != null)
				{
					// The ActiveCell may be displayed in multiple scroll regions. So 
					// we need to loop through all of them. 
					foreach (RowScrollRegion rsr in this.grid.DisplayLayout.RowScrollRegions)
					{
						foreach (ColScrollRegion csr in this.grid.DisplayLayout.ColScrollRegions)
						{
							UIElement activeCellUIElement = activeCell.GetUIElement(rsr, csr);
							if (activeCellUIElement != null)
							{								
								Rectangle activeCellRect = activeCellUIElement.ClipRect;
								// The cell is within the row, so we need to extend the rect a little
								// bit vertically. 
								activeCellRect.Inflate(0, 1);

								drawParams.DrawBorders(
									UIElementBorderStyle.Solid, 
									System.Windows.Forms.Border3DSide.All, 
									this.activeCellBorderColor, 
									this.activeCellBorderColor,
									activeCellRect, 
									drawParams.InvalidRect );								
							}
						}
					}
				}
			}

			return false;
		}
		#endregion

		#region Implementation of IDisposable
		public void Dispose()
		{
			if (this.grid != null)
				this.grid.BeforeCellDeactivate -= new CancelEventHandler(this.grid_BeforeCellDeactivate);
		}
		#endregion
	}
}
